unit Main;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, IdIOHandler, IdIOHandlerSocket, IdSSLOpenSSL, IdAntiFreezeBase,
  IdAntiFreeze, IdBaseComponent, IdComponent, IdTCPConnection, IdTCPClient,
  IdHTTP, ExtCtrls, StdCtrls, Buttons, Spin;

type
  TForm1 = class(TForm)
    Panel1: TPanel;
    edURL: TEdit;
    Button1: TButton;
    memoHTML: TMemo;
    Splitter1: TSplitter;
    logMemo: TMemo;
    HTTP: TIdHTTP;
    IdAntiFreeze1: TIdAntiFreeze;
    SSLIOHandler: TIdSSLIOHandlerSocket;
    txtRootCertificateFile: TEdit;
    Label1: TLabel;
    Label2: TLabel;
    txtKeyFile: TEdit;
    Label3: TLabel;
    Label4: TLabel;
    txtCertificateFile: TEdit;
    sbSelectFile: TSpeedButton;
    ofDialog: TOpenDialog;
    SpeedButton1: TSpeedButton;
    SpeedButton2: TSpeedButton;
    txtPassword: TEdit;
    Label5: TLabel;
    cmbMethod: TComboBox;
    Label6: TLabel;
    cbVerifyPeer: TCheckBox;
    cbVerifyNoPeerCert: TCheckBox;
    cbVerifyOnce: TCheckBox;
    seCertDepth: TSpinEdit;
    procedure Button1Click(Sender: TObject);
    function SSLIOHandlerVerifyPeer(
      Certificate: TIdX509): Boolean;
    procedure SSLIOHandlerGetPassword(var Password: String);
    procedure SSLIOHandlerStatusInfo(Msg: String);
    procedure FormDestroy(Sender: TObject);
    procedure sbSelectFileClick(Sender: TObject);
    procedure SpeedButton1Click(Sender: TObject);
    procedure SpeedButton2Click(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure cbVerifyPeerClick(Sender: TObject);
  private
    { Private declarations }
    AppPath: String;
    KeyPassword: String;
  public
    { Public declarations }
  end;

var
  Form1: TForm1;

implementation

{$R *.dfm}
uses IniFiles;

procedure TForm1.Button1Click(Sender: TObject);
begin
  Screen.Cursor := crHourGlass; try
    memoHTML.Clear;
    if edURL.Text ='' then begin
      ShowMessage('Specify a URL and a output file first!')
    end else begin
      SSLIOHandler.SSLOptions.RootCertFile := txtRootCertificateFile.Text;
      SSLIOHandler.SSLOptions.KeyFile := txtKeyFile.Text;
      SSLIOHandler.SSLOptions.CertFile := txtCertificateFile.Text;

      case cmbMethod.ItemIndex of
        // v2
        0: SSLIOHandler.SSLOptions.Method := sslvSSLv2;
        1: SSLIOHandler.SSLOptions.Method := sslvSSLv23;
        2: SSLIOHandler.SSLOptions.Method := sslvSSLv3;
        3: SSLIOHandler.SSLOptions.Method := sslvTLSv1;
      end;
      SSLIOHandler.SSLOptions.VerifyMode := [];
      if cbVerifyPeer.Checked then begin
        SSLIOHandler.SSLOptions.VerifyMode := [sslvrfPeer];
        if cbVerifyNoPeerCert.Checked then begin
          SSLIOHandler.SSLOptions.VerifyMode := SSLIOHandler.SSLOptions.VerifyMode + [sslvrfFailIfNoPeerCert];
        end;
        if cbVerifyOnce.Checked then begin
          SSLIOHandler.SSLOptions.VerifyMode := SSLIOHandler.SSLOptions.VerifyMode + [sslvrfClientOnce];
        end;
        SSLIOHandler.SSLOptions.VerifyDepth := seCertDepth.Value;
      end;

      KeyPassword := txtPassword.Text;

      memoHTML.Lines.Text := HTTP.Get(edURL.Text);
    end;
  finally Screen.Cursor := crDefault; end;
end;

function TForm1.SSLIOHandlerVerifyPeer(
  Certificate: TIdX509): Boolean;
begin
  logMemo.Lines.Add(DateTimeToStr(Certificate.notBefore));
  logMemo.Lines.Add(Certificate.Issuer.OneLine);
  logMemo.Lines.Add(Certificate.Subject.OneLine);
  Result := True;
end;

procedure TForm1.SSLIOHandlerGetPassword(var Password: String);
begin
  Password := KeyPassword;
end;

procedure TForm1.SSLIOHandlerStatusInfo(Msg: String);
begin
  logMemo.Lines.Add(Msg);
end;

procedure TForm1.FormDestroy(Sender: TObject);
var
  MyIniFile: TIniFile;
begin
  HTTP.DisconnectSocket;

  MyIniFile := TIniFile.Create(AppPath + 'sslclient.ini');
  MyIniFile.WriteString('SSLClient', 'URL', edURL.Text);
  MyIniFile.WriteString('SSLClient', 'Root Certificate', txtRootCertificateFile.Text);
  MyIniFile.WriteString('SSLClient', 'Personal Key', txtKeyFile.Text);
  MyIniFile.WriteString('SSLClient', 'Personal Certificate', txtCertificateFile.Text);

  MyIniFile.WriteString('SSLClient', 'Method', FormatFloat('0', cmbMethod.ItemIndex));

  if cbVerifyPeer.Checked then
    MyIniFile.WriteString('SSLClient', 'VerifyPeer', '1')
  else
    MyIniFile.WriteString('SSLClient', 'VerifyPeer', '0');

  if cbVerifyNoPeerCert.Checked then
    MyIniFile.WriteString('SSLClient', 'VerifyNoPeerCert', '1')
  else
    MyIniFile.WriteString('SSLClient', 'VerifyNoPeerCert', '0');

  if cbVerifyOnce.Checked then
    MyIniFile.WriteString('SSLClient', 'VerifyOnce', '1')
  else
    MyIniFile.WriteString('SSLClient', 'VerifyOnce', '0');


  MyIniFile.WriteString('SSLClient', 'CertDepth', FormatFloat('0', seCertDepth.Value));

  MyIniFile.Free;
end;

procedure TForm1.sbSelectFileClick(Sender: TObject);
begin
  ofDialog.FileName := '';
  ofDialog.Filter := 'Certificate (*.pem)|*.PEM|All files (*.*)|*.*';
  if ofDialog.Execute then begin
    txtRootCertificateFile.Text := ofDialog.FileName;
  end;
end;

procedure TForm1.SpeedButton1Click(Sender: TObject);
begin
  ofDialog.FileName := '';
  ofDialog.Filter := 'Certificate (*.pem)|*.PEM|All files (*.*)|*.*';
  if ofDialog.Execute then begin
    txtKeyFile.Text := ofDialog.FileName;
  end;
end;

procedure TForm1.SpeedButton2Click(Sender: TObject);
begin
  ofDialog.FileName := '';
  ofDialog.Filter := 'Certificate (*.pem)|*.PEM|All files (*.*)|*.*';
  if ofDialog.Execute then begin
    txtCertificateFile.Text := ofDialog.FileName;
  end;
end;

procedure TForm1.FormCreate(Sender: TObject);
var
  MyIniFile: TIniFile;
  s: String;
begin
  AppPath := ExtractFilePath(Application.ExeName);

  MyIniFile := TIniFile.Create(AppPath + 'sslclient.ini');
  edURL.Text := MyIniFile.ReadString('SSLClient', 'URL', 'https://www.intelicom.si');
  txtRootCertificateFile.Text := MyIniFile.ReadString('SSLClient', 'Root Certificate', '');
  txtKeyFile.Text := MyIniFile.ReadString('SSLClient', 'Personal Key', '');
  txtCertificateFile.Text := MyIniFile.ReadString('SSLClient', 'Personal Certificate', '');
  s := MyIniFile.ReadString('SSLClient', 'Method', '0');
  cmbMethod.ItemIndex := StrToInt(s);

  s := MyIniFile.ReadString('SSLClient', 'VerifyPeer', '0');
  if s = '1' then begin
    cbVerifyPeer.Checked := True;
    cbVerifyNoPeerCert.Enabled := True;
    cbVerifyOnce.Enabled := True;
    s := MyIniFile.ReadString('SSLClient', 'VerifyNoPeerCert', '0');
    if s = '1' then begin
      cbVerifyNoPeerCert.Checked := True;
    end
    else begin
      cbVerifyNoPeerCert.Checked := False;
    end;
    s := MyIniFile.ReadString('SSLClient', 'VerifyOnce', '0');
    if s = '1' then begin
      cbVerifyOnce.Checked := True;
    end
    else begin
      cbVerifyOnce.Checked := False;
    end;
  end
  else begin
    cbVerifyNoPeerCert.Checked := False;
    cbVerifyOnce.Checked := False;
    cbVerifyNoPeerCert.Enabled := False;
    cbVerifyOnce.Enabled := False;
  end;

  s := MyIniFile.ReadString('SSLClient', 'CertDepth', '2');
  seCertDepth.Value := StrToInt(s);

  MyIniFile.Free;
end;

procedure TForm1.cbVerifyPeerClick(Sender: TObject);
begin
  if not cbVerifyPeer.Checked then begin
    cbVerifyNoPeerCert.Checked := False;
    cbVerifyOnce.Checked := False;
    cbVerifyNoPeerCert.Enabled := False;
    cbVerifyOnce.Enabled := False;
    seCertDepth.Enabled := False;
  end
  else begin
    cbVerifyNoPeerCert.Enabled := True;
    cbVerifyOnce.Enabled := True;
    seCertDepth.Enabled := True;
    seCertDepth.Value := 2;
  end;

end;

end.
